package com.frist008.nixlibrary.cursor;

import android.content.Context;
import android.database.Cursor;
import android.support.v4.widget.CursorAdapter;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import com.frist008.nixlibrary.NixAdapter;
import com.frist008.nixlibrary.NixCreatorHolder;
import com.frist008.nixlibrary.NixException;
import com.frist008.nixlibrary.NixHolder;
import com.frist008.nixlibrary.NixMultiCreatorHolder;
import com.frist008.nixlibrary.database.NixWhere;

@SuppressWarnings("UnusedDeclaration")
public abstract class NixMultiCursorAdapter extends CursorAdapter implements NixAdapter<Cursor> {

	private final NixMultiCreatorHolder typeItemArr = new NixMultiCreatorHolder();

	public NixMultiCursorAdapter(Context context) {
		this(context, null);
	}

	public NixMultiCursorAdapter(Context context, Cursor cursor) {
		super(context, cursor, false);

		fillTypeItemArr(typeItemArr);

		if (typeItemArr.getCreatorHolderSparseArray().size() == 0) {
			throw new NixException("SparseArray not valid length (maybe SparseArray == 0?)");
		}
	}

	protected abstract void fillTypeItemArr(NixMultiCreatorHolder typeItemArr);

	/**
	 * @param cursor - The cursor from which to get the data.
	 * @return An integer representing the type of View. Two views should share the same type if one
	 * can be converted to the other in getView. Note: Integers must be in the range 0 to
	 * getViewTypeCount - 1 else ArrayIndexOutOfBoundsException. IGNORE_ITEM_VIEW_TYPE can also be
	 * returned.
	 */
	protected abstract int getItemViewType(Cursor cursor);

	@Deprecated
	@Override
	public int getViewTypeCount() {
		return typeItemArr.getCreatorHolderSparseArray().size();
	}

	@Deprecated
	@Override
	public int getItemViewType(int position) {
		Cursor cursor = getCursor();
		return cursor.moveToPosition(position) ? getItemViewType(cursor) : IGNORE_ITEM_VIEW_TYPE;
	}

	@Deprecated
	@Override
	public View newView(Context context, Cursor cursor, ViewGroup parent) {
		NixCreatorHolder holder = typeItemArr.getCreatorHolderSparseArray()
		                                     .get(getItemViewType(cursor));
		return holder == null
		       ? new View(context)
		       : holder.newView(LayoutInflater.from(context), parent).getParentView();
	}

	@SuppressWarnings("unchecked")
	@Deprecated
	@Override
	public void bindView(View view, Context context, Cursor cursor) {
		NixHolder<NixMultiCursorAdapter, Cursor> holder
				= ((NixHolder<NixMultiCursorAdapter, Cursor>) view.getTag());
		holder.setId(cursor.getInt(cursor.getColumnIndex(NixWhere.NixCols.COLUMN_ID)));
		holder.setViewType(getItemViewType(cursor));
		holder.setPosition(cursor.getPosition());
		holder.setAdapter(this);
		holder.bindView(cursor);
	}

	@Override
	public Cursor getData() {
		return getCursor();
	}
}
