package com.frist008.nixlibrary.database;

import android.content.ContentValues;
import android.database.Cursor;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;

import org.json.JSONArray;
import org.json.JSONException;

import java.util.ArrayList;
import java.util.List;

@SuppressWarnings("unused")
public abstract class NixColumnType<NixType extends NixColumnType, ValueType> {
	static final String INTEGER = " INTEGER ";
	static final String REAL = " REAL ";

	final NixTableOrViewBean bean;
	private final String columnName;

	private ValueType value;
	@Nullable
	private final ValueType defaultValue;
	private boolean nullableValue;
	private final boolean index;
	private final boolean unique;

	public NixColumnType(@NonNull NixTableOrViewBean bean, @NonNull String columnName) {
		this(bean, columnName, null, false, false);
	}

	public NixColumnType(@NonNull NixTableOrViewBean bean,
	                     @NonNull String columnName,
	                     boolean unique) {
		this(bean, columnName, null, unique, false);
	}

	public NixColumnType(@NonNull NixTableOrViewBean bean,
	                     @NonNull String columnName,
	                     boolean unique,
	                     boolean index) {
		this(bean, columnName, null, unique, index);
	}

	public NixColumnType(@NonNull NixTableOrViewBean bean,
	                     @NonNull String columnName,
	                     @Nullable ValueType defaultValue) {
		this(bean, columnName, defaultValue, false, false);
	}

	public NixColumnType(@NonNull NixTableOrViewBean bean,
	                     @NonNull String columnName,
	                     @Nullable ValueType defaultValue,
	                     boolean unique) {
		this(bean, columnName, defaultValue, unique, false);
	}

	public NixColumnType(@NonNull NixTableOrViewBean bean,
	                     @NonNull String columnName,
	                     @Nullable ValueType defaultValue,
	                     boolean unique,
	                     boolean index) {
		this.bean = bean;
		this.columnName = columnName;
		this.unique = unique;
		this.defaultValue = defaultValue;
		this.index = index;
		bean.putColumn(this);
	}

	protected boolean isNotNull() {
		return false;
	}

	/**
	 * If you make insert or update and call this method then column receive value null in Table.
	 */
	public void setNullableValue() {
		setNullableValue(true);
	}

	/**
	 * If you make insert or update and call this method then column with "nullableValue=true"
	 * receive value null in Table.
	 */
	public void setNullableValue(boolean nullableValue) {
		this.nullableValue = nullableValue;
	}

	@SuppressWarnings("unchecked")
	public NixType set(@Nullable ValueType value) {
		if (value == null) {
			setNullableValue();
		} else {
			this.value = value;
		}

		return (NixType) this;
	}

	@Override
	public String toString() {
		return columnName;
	}

	public ValueType get() {
		return value;
	}

	@Nullable
	public abstract ValueType getValue(Cursor cursor, int columnIndexOrThrow);

	void fillContentValues(ContentValues contentValues) {
		if (value != null || isNullableValue()) {
			fillContentValue(contentValues);
		}
	}

	abstract void fillContentValue(ContentValues contentValues);

	public void update(ValueType value) {
		update(value, null);
	}

	public void update(ValueType value, String selection, String... args) {
		if (value == null) {
			setNullableValue();
		} else {
			set(value);
		}

		ContentValues contentValues = new ContentValues();
		fillContentValue(contentValues);
		((NixTable) bean.getTableOrView()).updateRow(contentValues, selection, args);
	}

	/**
	 * If you make insert or update then column with "nullableValue=true" receive value null in
	 * Table.
	 */
	public boolean isNullableValue() {
		return nullableValue;
	}

	public boolean isEmptyValue() {
		return value == null;
	}

	public boolean hasValue() {
		return value != null;
	}

	@NonNull
	public ValueType[] getArray() {
		return getArray(bean, null);
	}

	@NonNull
	public ValueType[] getArray(@NonNull NixViewBean bean) {
		return getArray(bean, null);
	}

	@NonNull
	private ValueType[] getArray(@NonNull NixTableOrViewBean bean) {
		return getArray(bean, null);
	}

	@NonNull
	public ValueType[] getArray(@Nullable Sort sort) {
		return getArray(bean, null, sort, null);
	}

	@NonNull
	public ValueType[] getArray(@NonNull NixViewBean bean, @Nullable Sort sort) {
		return getArray(bean, null, sort, null);
	}

	@NonNull
	private ValueType[] getArray(@NonNull NixTableOrViewBean bean, @Nullable Sort sort) {
		return getArray(bean, null, sort, null);
	}

	@NonNull
	public ValueType[] getArray(@Nullable String selection, @Nullable String... args) {
		return getArray(bean, null, null, selection, args);
	}

	@NonNull
	public ValueType[] getArray(@NonNull NixViewBean bean,
	                            @Nullable String selection,
	                            @Nullable String... args) {
		return getArray(bean, null, null, selection, args);
	}

	@NonNull
	private ValueType[] getArray(@NonNull NixTableOrViewBean bean,
	                             @Nullable String selection,
	                             @Nullable String... args) {
		return getArray(bean, null, null, selection, args);
	}

	@NonNull
	public ValueType[] getArray(@Nullable String[] projection,
	                            @Nullable String selection,
	                            @Nullable String... args) {
		return getArray(bean, projection, null, selection, args);
	}

	@NonNull
	public ValueType[] getArray(@NonNull NixViewBean bean,
	                            @Nullable String[] projection,
	                            @Nullable String selection,
	                            @Nullable String... args) {
		return getArray(bean, projection, null, selection, args);
	}

	@NonNull
	private ValueType[] getArray(@NonNull NixTableOrViewBean bean,
	                             @Nullable String[] projection,
	                             @Nullable String selection,
	                             @Nullable String... args) {
		return getArray(bean, projection, null, selection, args);
	}

	@NonNull
	public ValueType[] getArray(@Nullable Sort sort,
	                            @Nullable String selection,
	                            @Nullable String... args) {
		return getArray(bean, null, sort, selection, args);
	}

	@NonNull
	public ValueType[] getArray(@NonNull NixViewBean bean,
	                            @Nullable Sort sort,
	                            @Nullable String selection,
	                            @Nullable String... args) {
		return getArray(bean, null, sort, selection, args);
	}

	@NonNull
	private ValueType[] getArray(@NonNull NixTableOrViewBean bean,
	                             @Nullable Sort sort,
	                             @Nullable String selection,
	                             @Nullable String... args) {
		return getArray(bean, null, sort, selection, args);
	}

	@NonNull
	public ValueType[] getArray(@Nullable String[] projection,
	                            @Nullable Sort sort,
	                            @Nullable String selection,
	                            @Nullable String... args) {
		return getArray(bean, null, sort, selection, args);
	}

	@NonNull
	public ValueType[] getArray(@NonNull NixViewBean bean,
	                            @Nullable String[] projection,
	                            @Nullable Sort sort,
	                            @Nullable String selection,
	                            @Nullable String... args) {
		return getArray((NixTableOrViewBean) bean, null, sort, selection, args);
	}

	@NonNull
	private ValueType[] getArray(@NonNull NixTableOrViewBean bean,
	                             @Nullable String[] projection,
	                             @Nullable Sort sort,
	                             @Nullable String selection,
	                             @Nullable String... args) {
		Cursor cursor = getCursor(bean, projection, sort, selection, args);
		ValueType[] result = newArrayForValues(cursor.getCount());

		try {
			if (cursor.moveToFirst()) {
				int columnId = cursor.getColumnIndex(toString());
				do {
					result[cursor.getPosition()] = getValue(cursor, columnId);
				} while (cursor.moveToNext());
			}
		} finally {
			cursor.close();
		}

		return result;
	}

	abstract ValueType[] newArrayForValues(int count);

	@NonNull
	public List<ValueType> getList() {
		return getList(bean, null);
	}

	@NonNull
	public List<ValueType> getList(@NonNull NixViewBean bean) {
		return getList(bean, null);
	}

	@NonNull
	private List<ValueType> getList(@NonNull NixTableOrViewBean bean) {
		return getList(bean, null);
	}

	@NonNull
	public List<ValueType> getList(@Nullable Sort sort) {
		return getList(bean, null, sort, null);
	}

	@NonNull
	public List<ValueType> getList(@NonNull NixViewBean bean, @Nullable Sort sort) {
		return getList(bean, null, sort, null);
	}

	@NonNull
	private List<ValueType> getList(@NonNull NixTableOrViewBean bean, @Nullable Sort sort) {
		return getList(bean, null, sort, null);
	}

	@NonNull
	public List<ValueType> getList(@Nullable String selection, @Nullable String... args) {
		return getList(bean, null, null, selection, args);
	}

	@NonNull
	public List<ValueType> getList(@NonNull NixViewBean bean,
	                               @Nullable String selection,
	                               @Nullable String... args) {
		return getList(bean, null, null, selection, args);
	}

	@NonNull
	private List<ValueType> getList(@NonNull NixTableOrViewBean bean,
	                                @Nullable String selection,
	                                @Nullable String... args) {
		return getList(bean, null, null, selection, args);
	}

	@NonNull
	public List<ValueType> getList(@Nullable String[] projection,
	                               @Nullable String selection,
	                               @Nullable String... args) {
		return getList(bean, projection, null, selection, args);
	}

	@NonNull
	public List<ValueType> getList(@NonNull NixViewBean bean,
	                               @Nullable String[] projection,
	                               @Nullable String selection,
	                               @Nullable String... args) {
		return getList(bean, projection, null, selection, args);
	}

	@NonNull
	private List<ValueType> getList(@NonNull NixTableOrViewBean bean,
	                                @Nullable String[] projection,
	                                @Nullable String selection,
	                                @Nullable String... args) {
		return getList(bean, projection, null, selection, args);
	}

	@NonNull
	public List<ValueType> getList(@Nullable Sort sort,
	                               @Nullable String selection,
	                               @Nullable String... args) {
		return getList(bean, null, sort, selection, args);
	}

	@NonNull
	public List<ValueType> getList(@NonNull NixViewBean bean,
	                               @Nullable Sort sort,
	                               @Nullable String selection,
	                               @Nullable String... args) {
		return getList(bean, null, sort, selection, args);
	}

	@NonNull
	private List<ValueType> getList(@NonNull NixTableOrViewBean bean,
	                                @Nullable Sort sort,
	                                @Nullable String selection,
	                                @Nullable String... args) {
		return getList(bean, null, sort, selection, args);
	}

	@NonNull
	public List<ValueType> getList(@Nullable String[] projection,
	                               @Nullable Sort sort,
	                               @Nullable String selection,
	                               @Nullable String... args) {
		return getList(bean, projection, sort, selection, args);
	}

	@NonNull
	private List<ValueType> getList(@NonNull NixViewBean bean,
	                                @Nullable String[] projection,
	                                @Nullable Sort sort,
	                                @Nullable String selection,
	                                @Nullable String... args) {
		return getList((NixTableOrViewBean) bean, projection, sort, selection, args);
	}

	@NonNull
	private List<ValueType> getList(@NonNull NixTableOrViewBean bean,
	                                @Nullable String[] projection,
	                                @Nullable Sort sort,
	                                @Nullable String selection,
	                                @Nullable String... args) {
		Cursor cursor = getCursor(bean, projection, sort, selection, args);
		List<ValueType> result = new ArrayList<>();

		try {
			if (cursor.moveToFirst()) {
				int columnId = cursor.getColumnIndex(toString());
				do {
					result.add(getValue(cursor, columnId));
				} while (cursor.moveToNext());
			}
		} finally {
			cursor.close();
		}

		return result;
	}

	@NonNull
	public JSONArray getJArray(@Nullable Sort sort) throws JSONException {
		return getJArray(bean, null, sort, null);
	}

	@NonNull
	public JSONArray getJArray(@NonNull NixViewBean bean,
	                           @Nullable Sort sort) throws JSONException {
		return getJArray(bean, null, sort, null);
	}

	@NonNull
	private JSONArray getJArray(@NonNull NixTableOrViewBean bean,
	                            @Nullable Sort sort) throws JSONException {
		return getJArray(bean, null, sort, null);
	}

	@NonNull
	public JSONArray getJArray(@Nullable String selection,
	                           @Nullable String... args) throws JSONException {
		return getJArray(bean, null, null, selection, args);
	}

	@NonNull
	public JSONArray getJArray(@NonNull NixViewBean bean,
	                           @Nullable String selection,
	                           @Nullable String... args) throws JSONException {
		return getJArray(bean, null, null, selection, args);
	}

	@NonNull
	private JSONArray getJArray(@NonNull NixTableOrViewBean bean,
	                            @Nullable String selection,
	                            @Nullable String... args) throws JSONException {
		return getJArray(bean, null, null, selection, args);
	}

	@NonNull
	public JSONArray getJArray(@Nullable String[] projection,
	                           @Nullable String selection,
	                           @Nullable String... args) throws JSONException {
		return getJArray(bean, projection, null, selection, args);
	}

	@NonNull
	public JSONArray getJArray(@NonNull NixViewBean bean,
	                           @Nullable String[] projection,
	                           @Nullable String selection,
	                           @Nullable String... args) throws JSONException {
		return getJArray(bean, projection, null, selection, args);
	}

	@NonNull
	private JSONArray getJArray(@NonNull NixTableOrViewBean bean,
	                            @Nullable String[] projection,
	                            @Nullable String selection,
	                            @Nullable String... args) throws JSONException {
		return getJArray(bean, projection, null, selection, args);
	}

	@NonNull
	public JSONArray getJArray(@Nullable Sort sort,
	                           @Nullable String selection,
	                           @Nullable String... args) throws JSONException {
		return getJArray(bean, null, sort, selection, args);
	}

	@NonNull
	public JSONArray getJArray(@NonNull NixViewBean bean,
	                           @Nullable Sort sort,
	                           @Nullable String selection,
	                           @Nullable String... args) throws JSONException {
		return getJArray(bean, null, sort, selection, args);
	}

	@NonNull
	private JSONArray getJArray(@NonNull NixTableOrViewBean bean,
	                            @Nullable Sort sort,
	                            @Nullable String selection,
	                            @Nullable String... args) throws JSONException {
		return getJArray(bean, null, sort, selection, args);
	}

	@NonNull
	public JSONArray getJArray(@Nullable String[] projection,
	                           @Nullable Sort sort,
	                           @Nullable String selection,
	                           @Nullable String... args) throws JSONException {
		return getJArray(bean, projection, sort, selection, args);
	}

	@NonNull
	public JSONArray getJArray(@NonNull NixViewBean bean,
	                           @Nullable String[] projection,
	                           @Nullable Sort sort,
	                           @Nullable String selection,
	                           @Nullable String... args) throws JSONException {
		return getJArray((NixTableOrViewBean) bean, projection, sort, selection, args);
	}

	@NonNull
	private JSONArray getJArray(@NonNull NixTableOrViewBean bean,
	                            @Nullable String[] projection,
	                            @Nullable Sort sort,
	                            @Nullable String selection,
	                            @Nullable String... args) throws JSONException {
		Cursor cursor = getCursor(bean, projection, sort, selection, args);
		JSONArray result = new JSONArray();

		try {
			if (cursor.moveToFirst()) {
				int columnId = cursor.getColumnIndex(toString());
				do {
					result.put(getValue(cursor, columnId));
				} while (cursor.moveToNext());
			}
		} finally {
			cursor.close();
		}

		return result;
	}

	@Nullable
	public ValueType getValue(@Nullable Sort sort) {
		return getValue(bean, null, sort, null);
	}

	@NonNull
	public ValueType getNotNullCustomValue(@Nullable ValueType defValue, @Nullable Sort sort) {
		return getNotNullCustomValue(defValue, bean, null, sort, null);
	}

	@NonNull
	public ValueType getNotNullValue(@Nullable Sort sort) {
		return getNotNullCustomValue(null, bean, null, sort, null);
	}

	@Nullable
	public ValueType getValue(@NonNull NixViewBean bean, @Nullable Sort sort) {
		return getValue(bean, null, sort, null);
	}

	@NonNull
	public ValueType getNotNullCustomValue(@Nullable ValueType defValue,
	                                       @NonNull NixViewBean bean,
	                                       @Nullable Sort sort) {
		return getNotNullCustomValue(defValue, bean, null, sort, null);
	}

	@NonNull
	public ValueType getNotNullValue(@NonNull NixViewBean bean, @Nullable Sort sort) {
		return getNotNullCustomValue(null, bean, null, sort, null);
	}

	@Nullable
	private ValueType getValue(@NonNull NixTableOrViewBean bean, @Nullable Sort sort) {
		return getValue(bean, null, sort, null);
	}

	@Nullable
	public ValueType getValue(@Nullable String selection, @Nullable String... args) {
		return getValue(bean, null, null, selection, args);
	}

	@NonNull
	public ValueType getNotNullCustomValue(@Nullable ValueType defValue,
	                                       @Nullable String selection,
	                                       @Nullable String... args) {
		return getNotNullCustomValue(defValue, bean, null, null, selection, args);
	}

	@NonNull
	public ValueType getNotNullValue(@Nullable String selection, @Nullable String... args) {
		return getNotNullCustomValue(null, bean, null, null, selection, args);
	}

	@Nullable
	public ValueType getValue(@NonNull NixViewBean bean,
	                          @Nullable String selection,
	                          @Nullable String... args) {
		return getValue(bean, null, null, selection, args);
	}

	@NonNull
	public ValueType getNotNullCustomValue(@Nullable ValueType defValue,
	                                       @NonNull NixViewBean bean,
	                                       @Nullable String selection,
	                                       @Nullable String... args) {
		return getNotNullCustomValue(defValue, bean, null, null, selection, args);
	}

	@NonNull
	public ValueType getNotNullValue(@NonNull NixViewBean bean,
	                                 @Nullable String selection,
	                                 @Nullable String... args) {
		return getNotNullCustomValue(null, bean, null, null, selection, args);
	}

	@Nullable
	private ValueType getValue(@NonNull NixTableOrViewBean bean,
	                           @Nullable String selection,
	                           @Nullable String... args) {
		return getValue(bean, null, null, selection, args);
	}

	@Nullable
	public ValueType getValue(@Nullable String[] projection,
	                          @Nullable String selection,
	                          @Nullable String... args) {
		return getValue(bean, projection, null, selection, args);
	}

	@NonNull
	public ValueType getNotNullCustomValue(@Nullable ValueType defValue,
	                                       @Nullable String[] projection,
	                                       @Nullable String selection,
	                                       @Nullable String... args) {
		return getNotNullCustomValue(defValue, bean, projection, null, selection, args);
	}

	@NonNull
	public ValueType getNotNullValue(@Nullable String[] projection,
	                                 @Nullable String selection,
	                                 @Nullable String... args) {
		return getNotNullCustomValue(null, bean, projection, null, selection, args);
	}

	@Nullable
	public ValueType getValue(@NonNull NixViewBean bean,
	                          @Nullable String[] projection,
	                          @Nullable String selection,
	                          @Nullable String... args) {
		return getValue(bean, projection, null, selection, args);
	}

	@NonNull
	public ValueType getNotNullCustomValue(@Nullable ValueType defValue,
	                                       @NonNull NixViewBean bean,
	                                       @Nullable String[] projection,
	                                       @Nullable String selection,
	                                       @Nullable String... args) {
		return getNotNullCustomValue(defValue, bean, projection, null, selection, args);
	}

	@NonNull
	public ValueType getNotNullValue(@NonNull NixViewBean bean,
	                                 @Nullable String[] projection,
	                                 @Nullable String selection,
	                                 @Nullable String... args) {
		return getNotNullCustomValue(null, bean, projection, null, selection, args);
	}

	@Nullable
	private ValueType getValue(@NonNull NixTableOrViewBean bean,
	                           @Nullable String[] projection,
	                           @Nullable String selection,
	                           @Nullable String... args) {
		return getValue(bean, projection, null, selection, args);
	}

	@Nullable
	public ValueType getValue(@Nullable Sort sort,
	                          @Nullable String selection,
	                          @Nullable String... args) {
		return getValue(bean, null, sort, selection, args);
	}

	@NonNull
	public ValueType getNotNullCustomValue(@Nullable ValueType defValue,
	                                       @Nullable Sort sort,
	                                       @Nullable String selection,
	                                       @Nullable String... args) {
		return getNotNullCustomValue(defValue, bean, null, sort, selection, args);
	}

	@NonNull
	public ValueType getNotNullValue(@Nullable Sort sort,
	                                 @Nullable String selection,
	                                 @Nullable String... args) {
		return getNotNullCustomValue(null, bean, null, sort, selection, args);
	}

	@Nullable
	public ValueType getValue(@NonNull NixViewBean bean,
	                          @Nullable Sort sort,
	                          @Nullable String selection,
	                          @Nullable String... args) {
		return getValue(bean, null, sort, selection, args);
	}

	@NonNull
	public ValueType getNotNullCustomValue(@Nullable ValueType defValue,
	                                       @NonNull NixViewBean bean,
	                                       @Nullable Sort sort,
	                                       @Nullable String selection,
	                                       @Nullable String... args) {
		return getNotNullCustomValue(defValue, bean, null, sort, selection, args);
	}

	@NonNull
	public ValueType getNotNullValue(@NonNull NixViewBean bean,
	                                 @Nullable Sort sort,
	                                 @Nullable String selection,
	                                 @Nullable String... args) {
		return getNotNullCustomValue(null, bean, null, sort, selection, args);
	}

	@Nullable
	private ValueType getValue(@NonNull NixTableOrViewBean bean,
	                           @Nullable Sort sort,
	                           @Nullable String selection,
	                           @Nullable String... args) {
		return getValue(bean, null, sort, selection, args);
	}

	@NonNull
	private ValueType getNotNullCustomValue(@Nullable ValueType defValue,
	                                        @NonNull NixTableOrViewBean bean,
	                                        @Nullable Sort sort,
	                                        @Nullable String selection,
	                                        @Nullable String... args) {
		return getNotNullCustomValue(defValue, bean, null, sort, selection, args);
	}

	@Nullable
	public ValueType getValue(@Nullable String[] projection,
	                          @Nullable Sort sort,
	                          @Nullable String selection,
	                          @Nullable String... args) {
		return getValue(bean, null, sort, selection, args);
	}

	@NonNull
	public ValueType getNotNullCustomValue(@Nullable ValueType defValue,
	                                       @Nullable String[] projection,
	                                       @Nullable Sort sort,
	                                       @Nullable String selection,
	                                       @Nullable String... args) {
		return getNotNullCustomValue(defValue, bean, null, sort, selection, args);
	}

	@NonNull
	public ValueType getNotNullValue(@Nullable String[] projection,
	                                 @Nullable Sort sort,
	                                 @Nullable String selection,
	                                 @Nullable String... args) {
		return getNotNullCustomValue(null, bean, null, sort, selection, args);
	}

	@Nullable
	public ValueType getValue(@NonNull NixViewBean bean,
	                          @Nullable String[] projection,
	                          @Nullable Sort sort,
	                          @Nullable String selection,
	                          @Nullable String... args) {
		return getValue((NixTableOrViewBean) bean, null, sort, selection, args);
	}

	@NonNull
	public ValueType getNotNullCustomValue(@Nullable ValueType defValue,
	                                       @NonNull NixViewBean bean,
	                                       @Nullable String[] projection,
	                                       @Nullable Sort sort,
	                                       @Nullable String selection,
	                                       @Nullable String... args) {
		return getNotNullCustomValue(defValue,
		                             (NixTableOrViewBean) bean,
		                             null,
		                             sort,
		                             selection,
		                             args);
	}

	@NonNull
	public ValueType getNotNullValue(@NonNull NixViewBean bean,
	                                 @Nullable String[] projection,
	                                 @Nullable Sort sort,
	                                 @Nullable String selection,
	                                 @Nullable String... args) {
		return getNotNullCustomValue(null, (NixTableOrViewBean) bean, null, sort, selection, args);
	}

	@NonNull
	private ValueType getNotNullCustomValue(@Nullable ValueType defValue,
	                                        @NonNull NixTableOrViewBean bean,
	                                        @Nullable String[] projection,
	                                        @Nullable Sort sort,
	                                        @Nullable String selection,
	                                        @Nullable String... args) {
		ValueType result = getValue(bean, projection, sort, selection, args);
		return result == null ? (defValue == null ? getStandardDefaultValue() : defValue) : result;
	}

	@Nullable
	private ValueType getValue(@NonNull NixTableOrViewBean bean,
	                           @Nullable String[] projection,
	                           @Nullable Sort sort,
	                           @Nullable String selection,
	                           @Nullable String... args) {
		Cursor cursor = getCursor(bean, projection, sort, selection, args);
		ValueType result = null;

		try {
			if (cursor.moveToFirst()) {
				result = getValue(cursor, cursor.getColumnIndex(toString()));
			}
		} finally {
			cursor.close();
		}

		return result;
	}

	@SuppressWarnings("ConstantConditions")
	@NonNull
	public Cursor getCursor(@Nullable String[] projection,
	                        @Nullable Sort sort,
	                        @Nullable String selection,
	                        @Nullable String... args) {
		return getCursor(bean, projection, sort, selection, args);
	}

	@SuppressWarnings("ConstantConditions")
	@NonNull
	public static Cursor getCursor(@NonNull NixTableOrViewBean bean,
	                               @Nullable String[] projection,
	                               @Nullable Sort sort,
	                               @Nullable String selection,
	                               @Nullable String... args) {
		return NixProvider.getContentResolver()
		                  .query(bean.getTableOrView().getUri(),
		                         projection,
		                         selection,
		                         args,
		                         sort != null ? sort.getOrderBy() : null);
	}

	@Nullable
	public ValueType getValue(Cursor cursor) {
		return getValue(cursor, false);
	}

	@Nullable
	public ValueType getValue(Cursor cursor, boolean close) {
		ValueType value = null;

		if (!cursor.isClosed()) {
			if (cursor.getCount() > 0) {
				if (cursor.getPosition() == -1) {
					cursor.moveToFirst();
				}

				value = getValue(cursor, cursor.getColumnIndex(toString()));
			}

			if (close) {
				cursor.close();
			}
		}

		return value;
	}

	public Long getMinLong() {
		return getMinLong(null);
	}

	public Long getMinLong(@Nullable String selection, @Nullable String... selectionArgs) {
		return bean.getTableOrView()
		           .getLongAggregateFunction(NixWhere.NixCols.MIN_ROW,
		                                     new String[]{NixWhere.min(columnName)},
		                                     selection,
		                                     selectionArgs);
	}

	public Double getMinDouble() {
		return getMinDouble(null);
	}

	public Double getMinDouble(@Nullable String selection, @Nullable String... selectionArgs) {
		return bean.getTableOrView()
		           .getDoubleAggregateFunction(NixWhere.NixCols.MIN_ROW,
		                                       new String[]{NixWhere.min(columnName)},
		                                       selection,
		                                       selectionArgs);
	}

	public Long getMaxLong() {
		return getMaxLong(null);
	}

	public Long getMaxLong(@Nullable String selection, @Nullable String... selectionArgs) {
		return bean.getTableOrView()
		           .getLongAggregateFunction(NixWhere.NixCols.MAX_ROW,
		                                     new String[]{NixWhere.max(columnName)},
		                                     selection,
		                                     selectionArgs);
	}

	public Double getMaxDouble() {
		return getMaxDouble(null);
	}

	public Double getMaxDouble(@Nullable String selection, @Nullable String... selectionArgs) {
		return bean.getTableOrView()
		           .getDoubleAggregateFunction(NixWhere.NixCols.MAX_ROW,
		                                       new String[]{NixWhere.max(columnName)},
		                                       selection,
		                                       selectionArgs);
	}

	public Long getSumLong(@Nullable String selection, @Nullable String... selectionArgs) {
		return bean.getTableOrView()
		           .getLongAggregateFunction(NixWhere.NixCols.SUM_ROW,
		                                     new String[]{NixWhere.sum(columnName)},
		                                     selection,
		                                     selectionArgs);
	}

	@SuppressWarnings("unchecked")
	public Double getSumDouble(@Nullable String selection, @Nullable String... args) {
		return bean.getTableOrView().getDoubleAggregateFunction(NixWhere.NixCols.SUM_ROW,
		                                                        new String[]{NixWhere.sum(columnName)},
		                                                        selection,
		                                                        args);
	}

	@SuppressWarnings("unchecked")
	public NixType set(NixType value) {
		return set((ValueType) value.get());
	}

	boolean isIndex() {
		return index;
	}

	public boolean isUnique() {
		return unique;
	}

	public boolean hasDefValue() {
		return defaultValue != null;
	}

	@Nullable
	public ValueType getDefaultValue() {
		return defaultValue;
	}

	@NonNull
	protected abstract ValueType getStandardDefaultValue();

	@Nullable
	abstract String getDBDefValue();

	public abstract String getType();
}
