package com.frist008.nixlibrary.database;

import android.content.ContentValues;
import android.database.Cursor;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;
import android.text.TextUtils;

import java.util.Locale;

@SuppressWarnings("unused")
public class NixStr extends NixColumnType<NixStr, String> {

	private static final char STR_WRAP = '\'';
	private static final String TEXT = " TEXT ";

	public NixStr(NixTableOrViewBean bean, String columnName) {
		super(bean, columnName);
	}

	public NixStr(@NonNull NixTableOrViewBean bean, @NonNull String columnName, boolean unique) {
		super(bean, columnName, unique);
	}

	public NixStr(@NonNull NixTableOrViewBean bean,
	              @NonNull String columnName,
	              boolean unique,
	              boolean index) {
		super(bean, columnName, unique, index);
	}

	public NixStr(@NonNull NixTableOrViewBean bean,
	              @NonNull String columnName,
	              @Nullable String defValue) {
		super(bean, columnName, defValue);
	}

	public NixStr(@NonNull NixTableOrViewBean bean,
	              @NonNull String columnName,
	              @Nullable String defValue,
	              boolean unique) {
		super(bean, columnName, defValue, unique);
	}

	public NixStr(@NonNull NixTableOrViewBean bean,
	              @NonNull String columnName,
	              @Nullable String defValue,
	              boolean unique,
	              boolean index) {
		super(bean, columnName, defValue, unique, index);
	}

	@Nullable
	@Override
	String getDBDefValue() {
		//noinspection ConstantConditions
		return hasDefValue() ? STR_WRAP + getDefaultValue() + STR_WRAP : null;
	}

	@Nullable
	@Override
	public String getValue(Cursor cursor, int columnIndexOrThrow) {
		return cursor.getString(columnIndexOrThrow);
	}

	@Nullable
	@Override
	public String get() {
		return super.get();
	}

	@Override
	void fillContentValue(ContentValues contentValues) {
		contentValues.put(toString(), get());
	}

	@Override
	public boolean isEmptyValue() {
		String value = super.get();
		return TextUtils.isEmpty(super.get()) || value.trim().length() == 0;
	}

	@Override
	public boolean hasValue() {
		String value = super.get();
		return !TextUtils.isEmpty(value) && value.trim().length() > 0;
	}

	@Override
	String[] newArrayForValues(int count) {
		return new String[count];
	}

	@NonNull
	@Override
	protected String getStandardDefaultValue() {
		return "";
	}

	@Override
	public String getType() {
		return TEXT;
	}

	@SuppressWarnings("ConstantConditions")
	public NixStr toLowerCase() {
		return set(isEmptyValue() ? get() : get().toLowerCase(Locale.getDefault()));
	}

	@SuppressWarnings("ConstantConditions")
	public NixStr toUpperCase() {
		return set(isEmptyValue() ? get() : get().toUpperCase(Locale.getDefault()));
	}

	@SuppressWarnings("ConstantConditions")
	public NixStr trim() {
		return set(isEmptyValue() ? get() : get().trim());
	}

	public void setNotEmpty(String str) {
		set(TextUtils.isEmpty(str) ? null : str);
	}
}
