package com.frist008.nixlibrary.database;

import android.content.ContentValues;
import android.net.Uri;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;

import java.lang.reflect.Array;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

@SuppressWarnings("unused")
public abstract class NixTableBean<Bean extends NixTableBean> extends NixTableOrViewBean<Bean> {

	public static final int NAME_ORIGINAL_ID = 0;

	public final NixId id = new NixId(this);

	@SuppressWarnings("ConstantConditions")
	@NonNull
	public Uri insert() {
		return NixProvider.getContentResolver()
		                  .insert(getTableOrView().getUri(), convertToContentValues());
	}

	public static void update(Collection<? extends NixTableBean> collection) {
		update(collection, null, (String) null);
	}

	/**
	 * @param collection - beans with data
	 * @param selection  - "_id = ?" - it select where symbol '?' will be replaced value from beans.
	 *                   Example getting value from bean beans[i].getArgs(colArr[i])
	 * @param colArr     - "_id" - it name column is key for getting value from beans
	 */
	public static void update(@NonNull Collection<? extends NixTableBean> collection,
	                          @Nullable String selection,
	                          String... colArr) {
		if (collection.size() > 0) {
			if (colArr == null) {
				colArr = new String[0];
			}

			ContentValues[] cvArr = new ContentValues[collection.size()];
			NixTable table = fillContentValues(collection, cvArr);

			// todo applyBatch
			for (ContentValues cv : cvArr) {
				String[] args = new String[colArr.length];

				for (int i = 0; i < colArr.length; i++) {
					args[i] = cv.getAsString(colArr[i]);
				}

				table.updateRow(cv, selection, args);
			}
		}
	}

	public static void update(@NonNull Collection<? extends NixTableBean> collection,
	                          @Nullable String selection,
	                          String[]... argsArr) {
		if (collection.size() > 0) {
			if (argsArr == null || argsArr.length != collection.size()) {
				argsArr = new String[collection.size()][0];
			}

			ContentValues[] cvArr = new ContentValues[collection.size()];
			NixTable table = fillContentValues(collection, cvArr);

			// todo applyBatch
			for (int i = 0; i < cvArr.length; i++) {
				table.updateRow(cvArr[i], selection, argsArr[i]);
			}
		}
	}

	public static void update(@NonNull Collection<? extends NixTableBean> collection,
	                          @NonNull String[] selectionArr,
	                          String[]... argsArr) {
		if (collection.size() > 0) {
			if (argsArr == null
			    || argsArr.length != collection.size()
			    || selectionArr.length != collection.size()) {

				argsArr = new String[collection.size()][0];
			}

			ContentValues[] cvArr = new ContentValues[collection.size()];
			NixTable table = fillContentValues(collection, cvArr);

			// todo applyBatch
			for (int i = 0; i < cvArr.length; i++) {
				table.updateRow(cvArr[i], selectionArr[i], argsArr[i]);
			}
		}
	}

	public void update() {
		update(null);
	}

	public void update(String selection, String... args) {
		getTableOrView().updateRow(fillContentValues(), selection, args);
	}

	@SuppressWarnings({"ReturnOfNull", "ConstantConditions"})
	@NonNull
	private static NixTable fillContentValues(Collection<? extends NixTableBean> collection,
	                                          ContentValues[] cvArr) {
		Iterator<? extends NixTableBean> iterator = collection.iterator();

		for (int i = 0; iterator.hasNext(); i++) {
			cvArr[i] = iterator.next().fillContentValues();
		}

		iterator = collection.iterator();

		return iterator.hasNext() ? iterator.next().getTableOrView() : null;
	}

	public ContentValues fillContentValues() {
		ContentValues cv = new ContentValues();
		Map<String, NixColumnType> map = getColumnMap();

		for (Map.Entry<String, NixColumnType> columnEntry : map.entrySet()) {
			columnEntry.getValue().fillContentValue(cv);
		}

		return cv;
	}

	@SuppressWarnings("unchecked")
	public static <Bean extends NixTableBean> int insert(@Nullable Bean... beanArr) {
		int result = 0;

		if (beanArr != null && beanArr.length > 0) {
			ContentValues[] contentValuesArr = new ContentValues[beanArr.length];

			for (int i = 0; i < contentValuesArr.length; i++) {
				contentValuesArr[i] = beanArr[i].convertToContentValues();
			}

			NixProvider.getContentResolver()
			           .bulkInsert(beanArr[0].getTableOrView().getUri(), contentValuesArr);
		}

		return result;
	}

	@SuppressWarnings("unchecked")
	public static <Bean extends NixTableBean> int insert(@NonNull List<Bean> beanCollection) {
		int result;

		if (beanCollection.isEmpty()) {
			result = 0;
		} else {
			Bean[] beanArr = (Bean[]) Array.newInstance(beanCollection.get(0).getClass(),
			                                            beanCollection.size());
			result = insert(beanCollection.toArray(beanArr));
		}

		return result;
	}

	public ContentValues convertToContentValues() {
		ContentValues contentValues = new ContentValues();

		for (NixColumnType columnType : columnMap.values()) {
			columnType.fillContentValues(contentValues);
		}

		return contentValues;
	}

	@Override
	@NonNull
	public abstract NixTable getTableOrView();
}
