package com.frist008.nixlibrary.database;

import android.database.Cursor;
import android.net.Uri;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;

import com.frist008.nixlibrary.utils.NixUri;

@SuppressWarnings("unused")
public abstract class NixTableOrView extends NixNameTable implements NixUri {

	private final boolean deprecated;

	public NixTableOrView(@NonNull String nameEntity, boolean deprecated) {
		super(nameEntity);
		this.deprecated = deprecated;
	}

	public abstract String getQueryForCreation();

	public String dot(String column) {
		return this + (NixWhere.DOT + column);
	}

	@SuppressWarnings("EqualsWhichDoesntCheckParameterClass")
	@Override
	public boolean equals(Object o) {
		return o.toString().equals(toString());
	}

	public boolean isDeprecated() {
		return deprecated;
	}

	public boolean isNotDeprecated() {
		return !deprecated;
	}

	@NonNull
	public Cursor getCursor() {
		return getCursor(null);
	}

	@NonNull
	public Cursor getCursor(@Nullable String selection, @Nullable String... args) {
		return getCursor(null, null, selection, args);
	}

	@NonNull
	public Cursor getCursor(@Nullable Sort sort,
	                        @Nullable String selection,
	                        @Nullable String... args) {
		return getCursor(null, sort, selection, args);
	}

	@SuppressWarnings("ConstantConditions")
	@NonNull
	public Cursor getCursor(@Nullable String[] projection,
	                        @Nullable Sort sort,
	                        @Nullable String selection,
	                        @Nullable String... args) {
		return NixProvider.getContentResolver()
		                  .query(getUri(),
		                         projection,
		                         selection,
		                         args,
		                         sort != null ? sort.getOrderBy() : null);
	}

	@NonNull
	public Uri addLastSegment(long id) {
		return Uri.withAppendedPath(getUri(), String.valueOf(id));
	}

	@SuppressWarnings("unchecked")
	Long getLongAggregateFunction(@NonNull String columnName,
	                              @NonNull String[] projection,
	                              @Nullable String selection,
	                              @Nullable String... selectionArgs) {
		AggregateFunction<Long> function = new AggregateFunction<Long>() {
			@Override
			Long getValue(Cursor cursor, int columnId) {
				return cursor.getLong(columnId);
			}
		}.queryValueAggregateFunction(getUri(), columnName, projection, selection, selectionArgs);

		return function.valueResult == null ? 0 : function.valueResult;
	}

	@SuppressWarnings("unchecked")
	Double getDoubleAggregateFunction(@NonNull String columnName,
	                                  @NonNull String[] projection,
	                                  @Nullable String selection,
	                                  @Nullable String... selectionArgs) {
		AggregateFunction<Double> function = new AggregateFunction<Double>() {
			@Override
			Double getValue(Cursor cursor, int columnId) {
				return cursor.getDouble(columnId);
			}
		}.queryValueAggregateFunction(getUri(), columnName, projection, selection, selectionArgs);

		return function.valueResult == null ? 0.0d : function.valueResult;
	}

	public long getCount() {
		return getLongAggregateFunction(NixWhere.NixCols.COUNT_ROW,
		                                new String[]{NixWhere.COUNT},
		                                null);
	}

	public Long getCount(@Nullable String selection,
	                     @Nullable String... selectionArgs) {
		return getLongAggregateFunction(NixWhere.NixCols.COUNT_ROW,
		                                new String[]{NixWhere.COUNT},
		                                selection,
		                                selectionArgs);
	}

	public boolean isEmpty(@Nullable String selection, @Nullable String... selectionArgs) {
		return getCount(selection, selectionArgs) == 0;
	}

	public boolean has(@Nullable String selection, @Nullable String... selectionArgs) {
		return getCount(selection, selectionArgs) > 0;
	}

	abstract static class AggregateFunction<Value extends Number> {
		@Nullable
		Value valueResult;

		AggregateFunction queryValueAggregateFunction(@NonNull Uri uri,
		                                              @NonNull String columnName,
		                                              @NonNull String[] projection,
		                                              @Nullable String selection,
		                                              @Nullable String... selectionArgs) {
			try {
				Cursor cursor = NixProvider.getContentResolver()
				                           .query(uri,
				                                  projection,
				                                  selection,
				                                  selectionArgs,
				                                  null);
				if (cursor != null) {
					if (cursor.moveToFirst()) {
						valueResult = getValue(cursor, cursor.getColumnIndex(columnName));
					}

					if (!cursor.isClosed()) {
						cursor.close();
					}
				}

				return this;
			} catch (@SuppressWarnings("ProhibitedExceptionCaught") NullPointerException e) {
				throw new RuntimeException("Your query must not contain group by.", e);
			}
		}

		abstract Value getValue(Cursor cursor, int columnId);
	}
}
