package com.frist008.nixlibrary.database;

import android.database.Cursor;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

@SuppressWarnings("unused")
public abstract class NixTableOrViewBean<Bean extends NixTableOrViewBean> {

	final Map<String, NixColumnType> columnMap = new HashMap<>();

	void putColumn(NixColumnType columnType) {
		columnMap.put(columnType.toString(), columnType);
	}

	/**
	 * Please not modify this Map
	 *
	 * @return All column
	 */
	public Map<String, NixColumnType> getColumnMap() {
		return columnMap;
	}

	@Nullable
	public Bean get(@NonNull Cursor cursor) {
		return get(cursor, false);
	}

	@Nullable
	@SuppressWarnings("unchecked")
	public Bean get(@Nullable Cursor cursor, boolean cursorClose) {
		Bean bean = null;

		if (cursor != null && !cursor.isClosed()) {
			if (cursor.getCount() > 0) {
				if (cursor.getPosition() == -1) {
					cursor.moveToFirst();
				}

				for (Map.Entry<String, NixColumnType> columnTypeEntry : columnMap.entrySet()) {
					int columnIndex = cursor.getColumnIndexOrThrow(columnTypeEntry.getKey());
					NixColumnType columnType = columnTypeEntry.getValue();
					columnType.set(columnType.getValue(cursor, columnIndex));
				}

				bean = (Bean) this;
			}

			if (cursorClose) {
				cursor.close();
			}
		}

		return bean;
	}

	@NonNull
	public abstract NixTableOrView getTableOrView();

	@Nullable
	public Bean get() {
		return get((String) null);
	}

	@Nullable
	public Bean get(@Nullable String selection,
	                @Nullable String... selectionArgs) {
		return get(null, null, selection, selectionArgs);
	}

	@Nullable
	public Bean get(@NonNull Sort sort,
	                @Nullable String selection,
	                @Nullable String... selectionArgs) {
		return get(null, sort, selection, selectionArgs);
	}

	@Nullable
	public Bean get(@Nullable String[] projection,
	                @Nullable String selection,
	                @Nullable String... selectionArgs) {
		return get(projection, null, selection, selectionArgs);
	}

	@Nullable
	public Bean get(@Nullable String[] projection,
	                @Nullable Sort sort,
	                @Nullable String selection,
	                @Nullable String... selectionArgs) {
		Cursor cursor = NixProvider.getContentResolver()
		                           .query(getTableOrView().getUri(),
		                                  projection,
		                                  selection,
		                                  selectionArgs,
		                                  sort == null ? null : sort.getOrderBy());
		Bean bean = null;

		if (cursor != null && !cursor.isClosed()) {
			if (cursor.moveToFirst()) {
				bean = get(cursor, true);
			}

			if (!cursor.isClosed()) {
				cursor.close();
			}
		}

		return bean;
	}

	@NonNull
	public List<Bean> getList() {
		return queryDBAndGetList(null, null, null);
	}

	@NonNull
	public List<Bean> getList(@Nullable String selection, @Nullable String... args) {
		return queryDBAndGetList(null, null, selection, args);
	}

	@NonNull
	public List<Bean> getList(@Nullable String[] projection,
	                          @Nullable String selection,
	                          @Nullable String... args) {
		return queryDBAndGetList(projection, null, selection, args);
	}

	@NonNull
	public List<Bean> getList(@NonNull Sort order) {
		return queryDBAndGetList(null, order, null);
	}

	@NonNull
	public List<Bean> getList(@Nullable Sort order,
	                          @Nullable String selection,
	                          @Nullable String... args) {
		return queryDBAndGetList(null, order, selection, args);
	}

	@NonNull
	public List<Bean> getList(@Nullable String[] projection,
	                          @Nullable Sort order,
	                          @Nullable String selection,
	                          @Nullable String... args) {
		return queryDBAndGetList(projection, order, selection, args);
	}

	@NonNull
	private List<Bean> queryDBAndGetList(@Nullable String[] projection,
	                                     @Nullable Sort sort,
	                                     @Nullable String selection,
	                                     @Nullable String... args) {
		Cursor cursor = NixProvider.getContentResolver()
		                           .query(getTableOrView().getUri(),
		                                  projection,
		                                  selection,
		                                  args,
		                                  sort == null ? null : sort.getOrderBy());
		List<Bean> list;

		if (cursor != null) {
			list = getList(cursor);

			if (!cursor.isClosed()) {
				cursor.close();
			}
		} else {
			list = new ArrayList<>(0);
		}

		return list;
	}

	@SuppressWarnings("unchecked")
	public List<Bean> getList(@NonNull Cursor cursor) {
		Bean bean = newEmptyBean();
		List<Bean> list = new ArrayList<>(cursor.getCount());

		if (cursor.moveToFirst()) {
			// todo maybe list
			Map<String, Integer> nameColumnMap = new HashMap<>(bean.columnMap.size());
			Set<String> nameColumnSet = bean.columnMap.keySet();

			for (String nameColumn : nameColumnSet) {
				nameColumnMap.put(nameColumn, cursor.getColumnIndex(nameColumn));
			}

			do {
				Set<Map.Entry<String, NixColumnType>> columnTypeEntrySet
						= bean.columnMap.entrySet();

				for (Map.Entry<String, NixColumnType> columnTypeEntry : columnTypeEntrySet) {
					int columnIndex = nameColumnMap.get(columnTypeEntry.getKey());
					NixColumnType columnType = columnTypeEntry.getValue();
					columnType.set(columnType.getValue(cursor, columnIndex));
				}

				list.add(bean);
				bean = newEmptyBean();

			} while (cursor.moveToNext());
		}

		return list;
	}

	@NonNull
	public abstract Bean newEmptyBean();
}
