package com.frist008.nixlibrary.database;

import android.content.UriMatcher;
import android.net.Uri;

import java.util.Collection;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.ConcurrentHashMap;

@SuppressWarnings("unused")
class NixUriMatcher extends UriMatcher {

	private final Map<String, Set<NixTableOrView>> uriMap = new ConcurrentHashMap<>();
	private int code = 0;

	public NixUriMatcher() {
		super(UriMatcher.NO_MATCH);
	}

	public Uri newUriTable(NixTable table) {
		Uri uri = parse(table);

		if (match(uri) == -1) {
			Set<NixTableOrView> uriSet = new HashSet<>();
			uriSet.add(table);
			uriMap.put(table.toString(), uriSet);
			addURI(table, NixProvider.getAuthorities());
		}

		return uri;
	}

	public Uri newUriView(NixView dbView, NixViewBean<? extends NixViewBean> viewBean) {
		Uri uri = parse(dbView);

		if (match(uri) == -1) {
			for (NixTableBean table : viewBean.tableList) {
				getTableUriSet(table.getTableOrView().toString()).add(dbView);
			}

			addURI(dbView, NixProvider.getAuthorities());
		}

		return uri;
	}

	private Uri parse(NixTableOrView tableOrView) {
		return Uri.parse("content://" + NixProvider.getAuthorities() + "/" + tableOrView);
	}

	private void addURI(NixTableOrView tableOrView, String authorities) {
		super.addURI(authorities, tableOrView + "/#", this.code++);
		super.addURI(authorities, tableOrView.toString(), this.code++);
	}

	protected Set<NixTableOrView> getTableUriSet(String nameTable) {
		return uriMap.get(nameTable);
	}

	protected Collection<Set<NixTableOrView>> values() {
		return uriMap.values();
	}

}
