package com.frist008.nixlibrary.database;

import android.net.Uri;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;
import android.text.TextUtils;

import com.frist008.nixlibrary.database.utils.NixSelectAllColumnBuilder;
import com.frist008.nixlibrary.database.utils.NixViewCreatorBuilder;
import com.frist008.nixlibrary.database.utils.NixViewFromBuilder;

@SuppressWarnings("unused")
public class NixView extends NixTableOrView {

	private final Uri contentUri;
	@NonNull
	private final NixViewBean<? extends NixViewBean> viewBean;
	@Nullable
	private final NixNameTableAs[] nameTableAsArr;

	public NixView(@NonNull NixViewBean<? extends NixViewBean> viewBean,
	               @Nullable NixNameTableAs... nameTableAsArr) {
		this(viewBean, false, nameTableAsArr);
	}

	public NixView(@NonNull NixViewBean<? extends NixViewBean> viewBean,
	               boolean deprecated,
	               @Nullable NixNameTableAs... nameTableAsArr) {
		this(viewBean, viewBean.getClass().getSimpleName(), deprecated, nameTableAsArr);
	}

	public NixView(@NonNull NixViewBean<? extends NixViewBean> viewBean,
	               @NonNull String nameView,
	               @Nullable NixNameTableAs... nameTableAsArr) {
		this(viewBean, nameView, false, nameTableAsArr);
	}

	public NixView(@NonNull NixViewBean<? extends NixViewBean> viewBean,
	               @NonNull String nameView,
	               boolean deprecated,
	               @Nullable NixNameTableAs... nameTableAsArr) {
		super(nameView, deprecated);
		this.viewBean = viewBean;
		this.nameTableAsArr = nameTableAsArr;
		this.contentUri = NixProvider.URI_MATCHER.newUriView(this, viewBean);
	}

	@Override
	public String getQueryForCreation() {
		NixViewFromBuilder fromBuilder = new NixViewFromBuilder();
		NixViewCreatorBuilder viewCreatedBuilder
				= new NixViewCreatorBuilder(this, newViewFieldBuilder(), fromBuilder);
		fillFrom(fromBuilder);
		String where = getWhere();

		if (!TextUtils.isEmpty(where)) {
			viewCreatedBuilder.where(where);
		}

		return viewCreatedBuilder.toString();
	}

	private NixSelectAllColumnBuilder newViewFieldBuilder() {
		final NixSelectAllColumnBuilder builder = new NixSelectAllColumnBuilder();

		for (NixTableBean bean : viewBean.tableList) {
			builder.append(bean.getTableOrView());
		}

		if (nameTableAsArr != null) {
			for (NixNameTableAs nameTableAs : nameTableAsArr) {
				builder.append(nameTableAs);
			}
		}

		return builder;
	}

	@Nullable
	protected String getWhere() {
		return null;
	}

	protected void fillFrom(NixViewFromBuilder fromBuilder) {
		for (NixTableBean tableBean : viewBean.tableList) {
			fromBuilder.tableOrView(tableBean.getTableOrView());
		}
	}

	@Override
	public Uri getUri() {
		return contentUri;
	}
}
