package com.frist008.nixlibrary.database;

import android.support.annotation.NonNull;
import android.support.annotation.Nullable;
import android.text.TextUtils;

import com.frist008.nixlibrary.NixException;
import com.frist008.nixlibrary.database.utils.BooleanInt;
import com.frist008.nixlibrary.database.utils.NixSelectAllColumnBuilder;
import com.frist008.nixlibrary.database.utils.NixUnionBuilder;
import com.frist008.nixlibrary.database.utils.NixViewFromBuilder;

import java.text.MessageFormat;
import java.util.Collection;

@SuppressWarnings("unused")
public abstract class NixWhere extends NixCoreWhere {

	private static final String WHERE = " WHERE ";
	public static final String ARG = " ? ";
	public static final String AND = " AND ";
	public static final String OR = " OR ";
	private static final String LIKE_COL = " LIKE ? ";
	private static final String LIKE_ARG_ANY = "%{0}%";
	private static final String LIKE_ARG_START = "%{0}";
	private static final String LIKE_ARG_END = "{0}%";
	private static final String LIKE_ARG = "{0}";
	public static final String EQUALS = " = ";
	public static final String NOT_EQUALS = " != ";
	public static final String LIMIT = " LIMIT ";
	public static final String LIMIT_ONE_ROW = LIMIT + "1 ";
	public static final String AS = " as ";
	public static final String ON = " ON ";
	protected static final String NULL = " NULL";
	public static final String IS_NULL = " IS NULL";
	public static final String IS_NOT_NULL = " IS NOT NULL";
	public static final String ORDER_BY = " ORDER BY ";
	public static final String GROUP_BY = " GROUP BY ";
	public static final String HAVING = " HAVING ";
	public static final String UNION = " UNION ";
	public static final String UNION_ALL = " UNION ALL ";
	public static final String COMMA_STR = ",";
	public static final char DOT = '.';
	public static final String DOT_STR = ".";
	private static final String IN = "{0} IN ({1})";
	private static final String NOT_IN = "{0} NOT IN ({1})";
	private static final String IN_SUBQUERY = "{0} IN {1}";
	private static final String NOT_IN_SUBQUERY = "{0} NOT IN {1}";
	public static final String LESS = " < ";
	public static final String GREATER = " > ";
	public static final String LESS_OR_EQUALS = " <= ";
	public static final String GREATER_OR_EQUALS = " >= ";
	private static final String SUM = " SUM({0}) ";
	private static final String MAX = " MAX({0}) ";
	private static final String MIN = " MIN({0}) ";
	public static final String COUNT = " COUNT(*) " + AS + NixCols.COUNT_ROW;
	private static final String CHECK = " CHECK({0})";
	@BooleanInt
	public static final int TRUE = BooleanInt.TRUE;
	@BooleanInt
	public static final int FALSE = BooleanInt.FALSE;
	public static final String TRUE_STR = "1";
	public static final String FALSE_STR = "0";
	public static final String EMPTY = "''";
	private static final String RAW_SUBQUERY = "(SELECT {0} FROM {1}";
	private static final String SUBQUERY = RAW_SUBQUERY + ')';
	private static final String SUBQUERY_WHERE = RAW_SUBQUERY + " WHERE {2})";
	private static final String SUBQUERY_GROUP_BY = RAW_SUBQUERY + GROUP_BY + "{2})";
	protected static final String JOIN = " JOIN ";
	public static final String LEFT_JOIN = " LEFT" + JOIN;
	public static final String INNER_JOIN = " INNER" + JOIN;

	public interface NixCols {
		String COLUMN_ID = "_id";
		String COUNT_ROW = "countRow";
		String SUM_ROW = "sumRow";
		String MAX_ROW = "maxRow";
		String MIN_ROW = "mixRow";
	}

	public static String and(String colOrCondition1, String colOrCondition2) {
		return colOrCondition1 + AND + colOrCondition2;
	}

	public static String andEqual(String colOrCondition1, String col) {
		return colOrCondition1 + AND + equal(col);
	}

	public static String andNotEqual(String colOrCondition1, String col) {
		return colOrCondition1 + AND + notEqual(col);
	}

	/**
	 * @param col as Cols.COUNT_ROW
	 */
	public static String count(String col) {
		return MessageFormat.format(COUNT, col);
	}

	/**
	 * @return Condition1 OR Condition2
	 */
	public static String or(String colOrCondition1, String colOrCondition2) {
		return colOrCondition1 + OR + colOrCondition2;
	}

	/**
	 * @return Condition1 OR column = ?
	 */
	public static String orEqual(String colOrCondition1, String col) {
		return colOrCondition1 + OR + equal(col);
	}

	/**
	 * @return Condition1 OR column != ?
	 */
	public static String orNotEqual(String colOrCondition1, String col) {
		return colOrCondition1 + OR + notEqual(col);
	}

	/**
	 * @return "colOrCondition"
	 */
	public static String str(Object value) {
		return '"' + value.toString() + '"';
	}

	/**
	 * @return (colOrCondition)
	 */
	public static String wrap(String colOrCondition) {
		return '(' + colOrCondition + ')';
	}

	/**
	 * @return column = ?
	 */
	public static String equal(String column) {
		return equal(column, ARG);
	}

	/**
	 * @return column != ?
	 */
	public static String notEqual(String column) {
		return notEqual(column, ARG);
	}

	/**
	 * @return column = {@link BooleanInt#TRUE}
	 */
	public static String equalsTrue(String column) {
		return equalValue(column, TRUE);
	}

	/**
	 * @return column != {@link BooleanInt#TRUE}
	 */
	public static String notEqualsTrue(String column) {
		return notEqualValue(column, TRUE);
	}

	/**
	 * @return column = {@link BooleanInt#FALSE}
	 */
	public static String equalsFalse(String column) {
		return equalValue(column, FALSE);
	}

	/**
	 * @return column != {@link BooleanInt#FALSE}
	 */
	public static String notEqualsFalse(String column) {
		return notEqualValue(column, FALSE);
	}

	/**
	 * @return column = ''
	 */
	public static String equalsEmpty(String column) {
		return equal(column, EMPTY);
	}

	/**
	 * @return column != ''
	 */
	public static String notEqualsEmpty(String column) {
		return notEqual(column, EMPTY);
	}

	/**
	 * @return column = value
	 */
	public static String equalValue(String column, Number value) {
		return column + EQUALS + value;
	}

	/**
	 * @return column = "value"
	 */
	public static String equalValue(String column, Object value) {
		return column + EQUALS + str(value);
	}

	/**
	 * @return column1 = column2
	 */
	public static String equal(String column1, String column2) {
		return column1 + EQUALS + column2;
	}

	/**
	 * @return column != value
	 */
	public static String notEqualValue(String column, Number value) {
		return column + NOT_EQUALS + value;
	}

	/**
	 * @return column != "value"
	 */
	public static String notEqualValue(String column, Object value) {
		return column + NOT_EQUALS + str(value);
	}

	/**
	 * @return column1 != column2
	 */
	public static String notEqual(String column1, String column2) {
		return column1 + NOT_EQUALS + column2;
	}

	/**
	 * @return column IS NOT NULL
	 */
	public static String notNull(String column) {
		return column + IS_NOT_NULL;
	}

	/**
	 * @return column IS NULL
	 */
	public static String nullable(String column) {
		return column + IS_NULL;
	}

	/**
	 * @return %value%
	 */
	public static String likeArgAny(CharSequence value) {
		return MessageFormat.format(LIKE_ARG_ANY, value);
	}

	/**
	 * @return %value
	 */
	public static String likeArgStart(CharSequence value) {
		return MessageFormat.format(LIKE_ARG_START, value);
	}

	/**
	 * @return value%
	 */
	public static String likeArgEnd(CharSequence value) {
		return MessageFormat.format(LIKE_ARG_END, value);
	}

	/**
	 * @return %value%
	 */
	public static String likeArg(CharSequence value) {
		return MessageFormat.format(LIKE_ARG, value);
	}

	/**
	 * @return column LIKE ?
	 */
	public static String likeCol(CharSequence col) {
		return col + LIKE_COL;
	}

	public static String subquery(NixColumnType type) {
		return MessageFormat.format(SUBQUERY, type, type.bean.getTableOrView());
	}

	public static String subqueryWhere(NixColumnType type, String where) {
		return MessageFormat.format(SUBQUERY_WHERE, type, type.bean.getTableOrView(), where);
	}

	public static String subqueryGroupBy(NixColumnType type, String groupBy) {
		return MessageFormat.format(SUBQUERY_GROUP_BY, type, type.bean.getTableOrView(), groupBy);
	}

	public static String subquery(String field, NixView view) {
		return MessageFormat.format(SUBQUERY, field, view);
	}

	public static String subqueryWhere(String field, NixView view, String where) {
		return MessageFormat.format(SUBQUERY_WHERE, field, view, where);
	}

	public static String subqueryGroupBy(String field, NixView view, String groupBy) {
		return MessageFormat.format(SUBQUERY_GROUP_BY, field, view, groupBy);
	}

	public static String inSubquery(String col, String subquery) {
		return MessageFormat.format(IN_SUBQUERY, col, subquery);
	}

	public static String notInSubquery(String col, String subquery) {
		return MessageFormat.format(NOT_IN_SUBQUERY, col, subquery);
	}

	public static String in(String col, String value, @Nullable String... valueArr) {
		return whereIn(col, IN, value, valueArr);
	}

	public static String notIn(String col, String value, @Nullable String... valueArr) {
		return whereIn(col, NOT_IN, value, valueArr);
	}

	public static String in(@NonNull String col, @NonNull String[] valueArr) {
		return whereIn(col, IN, valueArr);
	}

	public static String in(@NonNull String col, @NonNull Collection<String> valueList) {
		return whereIn(col, IN, valueList.toArray(new String[valueList.size()]));
	}

	public static String notIn(@NonNull String col, @NonNull String[] valueArr) {
		return whereIn(col, NOT_IN, valueArr);
	}

	public static String notIn(@NonNull String col, @NonNull Collection<String> valueList) {
		return whereIn(col, NOT_IN, valueList.toArray(new String[valueList.size()]));
	}

	private static String whereIn(@NonNull String col,
	                              @NonNull String whereIn,
	                              @NonNull String[] valueArr) {
		if (valueArr.length == 0) {
			throw new NixException(MessageFormat.format(whereIn, col, "null"));
		}

		StringBuilder builder = new StringBuilder();
		builder.append('"').append(valueArr[0]).append('"');

		for (int i = 1; i < valueArr.length; i++) {
			if (!TextUtils.isEmpty(valueArr[i])) {
				builder.append(COMMA).append('"').append(valueArr[i]).append('"');
			}
		}

		return MessageFormat.format(whereIn, col, builder);
	}

	private static String whereIn(@NonNull String col,
	                              @NonNull String whereIn,
	                              @NonNull String value,
	                              @Nullable String... valueArr) {
		StringBuilder builder = new StringBuilder();

		builder.append('"').append(value).append('"');

		if (valueArr != null) {
			for (String str : valueArr) {
				if (!TextUtils.isEmpty(str)) {
					builder.append(COMMA).append('"').append(str).append('"');
				}
			}
		}

		return MessageFormat.format(whereIn, col, builder);
	}

	public static String in(String col, Number value, @Nullable Number... valueArr) {
		return whereIn(col, IN, value, valueArr);
	}

	public static String notIn(String col, Number value, @Nullable Number... valueArr) {
		return whereIn(col, NOT_IN, value, valueArr);
	}

	public static String in(@NonNull String col, @NonNull Number[] valueArr) {
		return whereIn(col, IN, valueArr);
	}

	public static String notIn(@NonNull String col, @NonNull Number[] valueArr) {
		return whereIn(col, NOT_IN, valueArr);
	}

	private static String whereIn(@NonNull String col,
	                              @NonNull String whereIn,
	                              @NonNull Number[] valueArr) {
		if (valueArr.length == 0) {
			throw new NixException(MessageFormat.format(whereIn, col, "null"));
		}

		StringBuilder builder = new StringBuilder();
		builder.append(valueArr[0]);

		for (int i = 1; i < valueArr.length; i++) {
			builder.append(COMMA).append(valueArr[i]);
		}

		return MessageFormat.format(whereIn, col, builder);
	}

	private static String whereIn(@NonNull String col,
	                              @NonNull String whereIn,
	                              @NonNull Number value,
	                              @Nullable Number... valueArr) {
		StringBuilder builder = new StringBuilder();

		builder.append(value);

		if (valueArr != null) {
			for (Number number : valueArr) {
				builder.append(COMMA).append(number);
			}
		}

		return MessageFormat.format(whereIn, col, builder);
	}

	public static String simpleUnionAllWhere(NixView view) {
		return simpleUnionAll(view) + WHERE;
	}

	public static String simpleUnionAll(NixView view) {
		return new NixUnionBuilder(new NixSelectAllColumnBuilder().append(view),
		                           new NixViewFromBuilder().tableOrView(view)).toString();
	}

	public static String check(String str) {
		return MessageFormat.format(CHECK, str);
	}

	public static String sum(String nameCol) {
		return MessageFormat.format(SUM, nameCol) + AS + NixCols.SUM_ROW;
	}

	public static String max(String nameCol) {
		return MessageFormat.format(MAX, nameCol) + AS + NixCols.MAX_ROW;
	}

	public static String min(String nameCol) {
		return MessageFormat.format(MIN, nameCol) + AS + NixCols.MIN_ROW;
	}
}
