package com.frist008.nixlibrary.database.utils;

import android.support.annotation.NonNull;

import com.frist008.nixlibrary.database.NixWhere;

import java.text.MessageFormat;

@SuppressWarnings("unused")
public class NixUnionBuilder {

	private static final String UNION_ALL = NixWhere.UNION_ALL + "SELECT {0} " + "FROM {1} ";
	private static final String WHERE = " WHERE ";

	private final StringBuilder builder = new StringBuilder();

	@NonNull
	private final NixSelectAllColumnBuilder select;
	@NonNull
	private final NixViewFromBuilder from;

	public NixUnionBuilder(@NonNull NixSelectAllColumnBuilder select,
	                       @NonNull NixViewFromBuilder from) {
		this.select = select;
		this.from = from;
	}

	public NixUnionBuilder unionAll(@NonNull NixSelectAllColumnBuilder select,
	                                @NonNull NixViewFromBuilder from) {
		builder.append(MessageFormat.format(UNION_ALL, select, from));
		return this;
	}

	public NixUnionBuilder where(@NonNull String where) {
		builder.append(WHERE).append(where);
		return this;
	}

	public NixUnionBuilder groupBy(@NonNull String groupBy) {
		builder.append(NixWhere.GROUP_BY).append(groupBy);
		return this;
	}

	public NixUnionBuilder having(@NonNull String having) {
		builder.append(NixWhere.HAVING).append(having);
		return this;
	}

	@Override
	public String toString() {
		return MessageFormat.format(UNION_ALL, select, from) + builder;
	}

	@NonNull
	public NixSelectAllColumnBuilder getSelect() {
		return select;
	}

	@NonNull
	public NixViewFromBuilder getFrom() {
		return from;
	}

	public StringBuilder getBuilder() {
		return builder;
	}
}
