package com.frist008.nixlibrary.database.utils;

import com.frist008.nixlibrary.database.NixNameTable;
import com.frist008.nixlibrary.database.NixNameTableAs;
import com.frist008.nixlibrary.database.NixTableOrView;
import com.frist008.nixlibrary.database.NixWhere;

@SuppressWarnings("unused")
public class NixViewFromBuilder {

	private final StringBuilder builder = new StringBuilder();

	public NixViewFromBuilder() {
	}

	@Override
	public String toString() {
		return builder.toString();
	}

	public NixViewFromBuilder tableOrView(NixTableOrView table) {
		builder.append(builder.length() > 0 ? NixWhere.COMMA : "").append(table);
		return this;
	}

	public NixViewFromBuilder tableOrView(NixNameTableAs asTable) {
		builder.append(builder.length() > 0 ? NixWhere.COMMA : "").append(asTable.getOriginalName())
		       .append(NixWhere.AS).append(asTable);
		return this;
	}

	public NixViewJoinBuilder leftJoin(NixTableOrView table) {
		builder.append(NixWhere.LEFT_JOIN).append(table).append(NixWhere.ON);
		return new NixViewJoinBuilder();
	}

	public NixViewJoinBuilder leftJoin(NixNameTableAs asTable) {
		builder.append(NixWhere.LEFT_JOIN).append(asTable.getOriginalName())
		       .append(NixWhere.AS).append(asTable).append(NixWhere.ON);
		return new NixViewJoinBuilder();
	}

	public NixViewJoinBuilder innerJoin(NixTableOrView table) {
		builder.append(NixWhere.INNER_JOIN).append(table).append(NixWhere.ON);
		return new NixViewJoinBuilder();
	}

	public NixViewJoinBuilder innerJoin(NixNameTableAs asTable) {
		builder.append(NixWhere.INNER_JOIN).append(asTable.getOriginalName())
		       .append(NixWhere.AS).append(asTable).append(NixWhere.ON);
		return new NixViewJoinBuilder();
	}

	public class NixViewJoinBuilder {

		public NixViewJoinWhereBuilder col(NixNameTable table, String field) {
			builder.append(table).append(NixWhere.DOT).append(field);
			return new NixViewJoinWhereBuilder();
		}

		public NixViewJoinWhereBuilder colStartWrap(NixNameTable table, String field) {
			builder.append('(').append(table).append(NixWhere.DOT).append(field);
			return new NixViewJoinWhereBuilder();
		}

		public NixViewJoinBuilder custom(Object custom) {
			builder.append(custom);
			return this;
		}

		public NixViewFromBuilder endJoin() {
			return NixViewFromBuilder.this;
		}

		public class NixViewJoinWhereBuilder {

			public NixViewJoinWhereBuilder equal(NixNameTable table, String field) {
				builder.append(NixWhere.EQUALS);
				return NixViewJoinBuilder.this.col(table, field);
			}

			public NixViewJoinWhereBuilder equal(Object field) {
				builder.append(NixWhere.EQUALS).append(NixWhere.str(field));
				return this;
			}

			public NixViewJoinWhereBuilder equal(Number field) {
				builder.append(NixWhere.EQUALS).append(field);
				return this;
			}

			public NixViewJoinWhereBuilder startWrap() {
				builder.append('(');
				return this;
			}

			public NixViewJoinWhereBuilder endWrap() {
				builder.append(')');
				return this;
			}

			public NixViewJoinWhereBuilder notEqual(NixNameTable table, String field) {
				builder.append(NixWhere.NOT_EQUALS);
				return NixViewJoinBuilder.this.col(table, field);
			}

			public NixViewJoinWhereBuilder notEqual(Object field) {
				builder.append(NixWhere.NOT_EQUALS).append(field);
				return this;
			}

			public NixViewJoinWhereBuilder and(NixNameTable table, String field) {
				builder.append(NixWhere.AND);
				return NixViewJoinBuilder.this.col(table, field);
			}

			public NixViewJoinWhereBuilder or(NixNameTable table, String field) {
				builder.append(NixWhere.OR);
				return NixViewJoinBuilder.this.col(table, field);
			}

			public NixViewJoinWhereBuilder orStartWrap(NixNameTable table, String field) {
				builder.append(NixWhere.OR);
				return NixViewJoinBuilder.this.colStartWrap(table, field);
			}

			public NixViewJoinWhereBuilder equalsTrue() {
				builder.append(NixWhere.EQUALS).append(NixWhere.TRUE_STR);
				return this;
			}

			public NixViewJoinWhereBuilder notEqualsTrue() {
				builder.append(NixWhere.NOT_EQUALS).append(NixWhere.TRUE_STR);
				return this;
			}

			public NixViewJoinWhereBuilder equalsFalse() {
				builder.append(NixWhere.EQUALS).append(NixWhere.FALSE_STR);
				return this;
			}

			public NixViewJoinWhereBuilder notEqualsFalse() {
				builder.append(NixWhere.NOT_EQUALS).append(NixWhere.FALSE_STR);
				return this;
			}

			public NixViewJoinWhereBuilder equalsEmpty() {
				builder.append(NixWhere.EQUALS).append(NixWhere.EMPTY);
				return this;
			}

			public NixViewJoinWhereBuilder notEqualsEmpty() {
				builder.append(NixWhere.NOT_EQUALS).append(NixWhere.EMPTY);
				return this;
			}

			public NixViewJoinWhereBuilder nullable(NixNameTable table, String field) {
				col(table, field);
				builder.append(NixWhere.EQUALS);
				nullable();
				return this;
			}

			public NixViewJoinWhereBuilder notNull(NixNameTable table, String field) {
				col(table, field);
				builder.append(NixWhere.EQUALS);
				notNull();
				return this;
			}

			public NixViewJoinWhereBuilder nullable() {
				builder.append(NixWhere.IS_NULL);
				return this;
			}

			public NixViewJoinWhereBuilder notNull() {
				builder.append(NixWhere.IS_NOT_NULL);
				return this;
			}

			public NixViewJoinBuilder custom(Object custom) {
				builder.append(custom);
				return NixViewJoinBuilder.this;
			}

			public NixViewFromBuilder endJoin() {
				return NixViewJoinBuilder.this.endJoin();
			}

		}
	}
}
