package com.frist008.nixlibrary.recycler.cursor;

import android.database.Cursor;
import android.database.DataSetObserver;
import android.support.annotation.LayoutRes;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;
import android.support.v7.widget.RecyclerView;
import android.view.LayoutInflater;
import android.view.ViewGroup;

import com.frist008.nixlibrary.NixAdapter;
import com.frist008.nixlibrary.NixCreatorHolder;
import com.frist008.nixlibrary.NixHolder;
import com.frist008.nixlibrary.NixRecyclerViewHolder;
import com.frist008.nixlibrary.database.NixWhere;
import com.frist008.nixlibrary.recycler.CursorRecyclerAdapter;

@SuppressWarnings({"unused", "deprecation"})
public class NixCursorRecyclerAdapter extends RecyclerView.Adapter<NixRecyclerViewHolder>
		implements NixAdapter<Cursor>, CursorRecyclerAdapter {

	private final DataSetObserver dataSetObserver = new NotifyingDataSetObserver();
	@Nullable
	private Cursor cursor;
	private boolean cursorValid = false;
	private int rowIdColumn;
	private final NixCreatorHolder creatorHolder;

	public NixCursorRecyclerAdapter(@NonNull Class<? extends NixHolder> holderClass,
	                                @LayoutRes int layoutRes) {
		this.creatorHolder = new NixCreatorHolder(holderClass, layoutRes);
	}

	@Nullable
	public Cursor getCursor() {
		return cursor;
	}

	@Override
	public int getItemCount() {
		//noinspection ConstantConditions
		return cursorValid ? cursor.getCount() : 0;
	}

	@Override
	public long getItemId(int position) {
		//noinspection ConstantConditions
		return (cursorValid && cursor.moveToPosition(position))
		       ? cursor.getLong(rowIdColumn)
		       : 0;
	}

	@Override
	public void setHasStableIds(boolean hasStableIds) {
		super.setHasStableIds(true);
	}

	@SuppressWarnings("deprecation")
	@Deprecated
	public NixRecyclerViewHolder onCreateViewHolder(LayoutInflater inflater,
	                                                ViewGroup parent,
	                                                int viewType) {
		NixHolder holder = creatorHolder.newView(inflater, parent);
		return new NixRecyclerViewHolder(holder.getParentView(), holder);
	}

	@Deprecated
	@SuppressWarnings("unchecked")
	public void onBindViewHolder(NixHolder holder, Cursor cursor) {
		holder.setAdapter(this);
		holder.bindView(cursor);
	}

	@SuppressWarnings("deprecation")
	@Deprecated
	@Override
	public NixRecyclerViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {
		//noinspection deprecation
		return onCreateViewHolder(LayoutInflater.from(parent.getContext()), parent, viewType);
	}

	@Deprecated
	@SuppressWarnings("ConstantConditions")
	@Override
	public void onBindViewHolder(
			@SuppressWarnings("deprecation") NixRecyclerViewHolder recyclerHolder, int position) {
		if (cursor.moveToPosition(position)) {
			NixHolder holder = recyclerHolder.getHolder();
			holder.setId(cursor.getInt(cursor.getColumnIndex(NixWhere.NixCols.COLUMN_ID)));
			holder.setPosition(position);

			//noinspection deprecation
			onBindViewHolder(holder, cursor);
		}
	}

	/**
	 * Change the underlying cursor to a new cursor. If there is an existing cursor it will be
	 * closed.
	 */
	@Override
	public void changeCursor(@Nullable Cursor cursor) {
		Cursor old = swapCursor(cursor);

		if (old != null) {
			old.close();
		}
	}

	/**
	 * Swap in a new Cursor, returning the old Cursor.  Unlike {@link #changeCursor(Cursor)}, the
	 * returned old Cursor is <em>not</em> closed.
	 */
	@Override
	@Nullable
	public Cursor swapCursor(@Nullable Cursor newCursor) {
		if (cursor != null && cursor.equals(newCursor)) {
			return null;
		}

		final Cursor oldCursor = cursor;

		if (oldCursor != null) {
			oldCursor.unregisterDataSetObserver(dataSetObserver);
		}

		cursor = newCursor;

		if (cursorValid = cursor != null) {
			cursor.registerDataSetObserver(dataSetObserver);
			rowIdColumn = cursor.getColumnIndexOrThrow(NixWhere.NixCols.COLUMN_ID);
		} else {
			rowIdColumn = -1;
		}

		notifyDataSetChanged();

		return oldCursor;
	}

	@Override
	public Cursor getData() {
		return getCursor();
	}

	private class NotifyingDataSetObserver extends DataSetObserver {

		@Override
		public void onChanged() {
			super.onChanged();
			notifyDataSetChanged();
		}

		@Override
		public void onInvalidated() {
			super.onInvalidated();
			notifyDataSetChanged();
		}
	}
}
